// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997-2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	CalDataVector.h	--- classes representing a data vector
//
//	DESCRIPTION:
//		Some simple wrapper classes around pointers to integer/real vectors
//
//	CHANGE HISTORY:
//		UL, 1999-05-31: 1.0.0: - first version
//		UL, 1999-11-24: 1.1.0: - added CalInt32Vector
//		UL, 2001-12-11: 1.2.0: - bug fix in CalDataVector<IntType>::operator=()
//
//	AUTHOR:
//		U. Lammers, Mon May 31 16:45:25 MET DST 1999
//
//	REFERENCES:
//		[1]: Software Specification Document for the XMM Science Analysis
//			 Subsystem, M. Dahlem et. al.,
//
//	$Header: /sasbuild/sas_git_repo/cvs_copy/sasdev/sas/packages/cal/src/CalDataVector.h,v 1.18 2009/07/06 14:25:19 aibarra Exp $
//

#ifndef CAL_DATAVECTOR_H
#define CAL_DATAVECTOR_H

#include "Dal.h"
#include "CalChild.h"

class CalServer;

template<class ElementType>
class CalVector {
public:
	CalVector(ElementType *ptr = 0, unsigned n = 0);
	//	copy constructor (performs shallow copy) / assignment operator
	CalVector(const CalVector<ElementType> &src);
	CalVector<ElementType> &operator=(const CalVector<ElementType> &rhs);

	inline ElementType &operator[](unsigned n) const { return _ptr[n]; }
	inline ElementType *ptr() const { return _ptr; }
	inline unsigned size() const { return _n; }

	//	assignment from scalar
	inline CalVector<ElementType> &operator=(ElementType v) 
               { for(unsigned n=0; n<_n; _ptr[n++] = v){}; return *this; }

	//	do a deep copy
	void clone(const CalVector<ElementType> &source);
	CalVector<ElementType> clone() const { CalVector<ElementType> v;
											v.clone(*this); return v; }

	virtual ~CalVector();

protected:
	ElementType *_ptr;
	unsigned    _n;
	unsigned	*_pcount;
}; // CalVector()


template<class RealType> class CalRealVector;

template<class IntType>
class CalIntVector : public CalVector<IntType>,
					 public CalChild<CalServer> {
public:
	CalIntVector(IntType *ptr = 0, unsigned n = 0,
				 const CalServer *server = 0)
		: CalVector<IntType>(ptr, n), CalChild<CalServer>(server) {}

	CalIntVector(const CalIntVector<IntType> &src)
		: CalVector<IntType>(src), CalChild<CalServer>(0) {}

	CalIntVector<IntType> &operator=(const CalIntVector<IntType> &rhs) {
		CalVector<IntType>::operator=(rhs); return *this; }

	CalIntVector &operator=(IntType v) {
		CalVector<IntType>::operator=(v); return *this; }

	CalRealVector<real32> &toReal(CalRealVector<real32> &rv) const;

	~CalIntVector() {}
}; // CalIntVector


template<class RealType>
class CalRealVector : public CalVector<RealType>,
					  public CalChild<CalServer> {
public:
	CalRealVector(RealType *ptr = 0, unsigned n = 0,
				  const CalServer *server = 0)
		: CalVector<RealType>(ptr, n), CalChild<CalServer>(server) {}

	CalRealVector(const CalRealVector<RealType> &src)
		: CalVector<RealType>(src), CalChild<CalServer>(0) {}

	CalRealVector<RealType> &operator=(const CalRealVector<RealType> &rhs) {
		CalVector<RealType>::operator=(rhs); return *this; }

	CalRealVector<RealType> clone() const { CalRealVector<RealType> v;
											CalVector<RealType> &vc = v;
											vc.clone(*this); return v; }

	template<class ToIntType>
	CalIntVector<ToIntType> &toInt(CalIntVector<ToIntType> &iv) const;

	~CalRealVector() {}
}; // CalRealVector

typedef CalIntVector<int8>				CalInt8Vector;
typedef CalIntVector<int16>             CalInt16Vector;
typedef CalIntVector<int32>             CalInt32Vector;
typedef CalRealVector<real32>           CalReal32Vector;
typedef CalRealVector<real64>			CalReal64Vector;

#endif
