// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997-2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	ContaminantDataServer.cc --- implementation of ContaminantDataServer class
//
//	DESCRIPTION:
//		This implements all methods declared in the ContaminantDataServer class
//
//	CHANGE HISTORY:
//		RS, 2013-7-23: 1.0.0: - first version
//
//	AUTHOR:
//		R. Saxton (RS), Tue July 23rd 2013
//
//	REFERENCES:
//		[1]: Software Specification Document for the XMM Science Analysis
//			 Subsystem, M. Dahlem et. al.,
//
//	$Header: /sasbuild/sas_git_repo/cvs_copy/sasdev/sas/packages/cal/src/ContaminantDataServer.cc,v 1.1 2013/08/23 11:08:12 rsaxton Exp $
//

#include <cmath>
#include "CalOalUtility.h"
#include "CalServer.h"
#include "ContaminantDataServer.h"

using namespace CalOalUtility;
using namespace std;

//
//	---( ContaminantTransmissionCurve )-------------------------------------------
//
double ContaminantTransmissionCurve::transmission(double E) const
{
	return (*this)(E);
} // ContaminantTransmissionCurve::transmission()


double ContaminantTransmissionCurve::transmission(double Elo, double Ehi) const
{
	if (fabs(Ehi-Elo) < 1.e-8)
		return this->transmission(Elo);

	return this->integral(Elo, Ehi)/(Ehi-Elo);
} // ContaminantTransmissionCurve::transmission


//
//	---( ContaminantDataServer )--------------------------------------------------
//
const ContaminantTransmissionCurve ContaminantDataServer::_zeroCurve(0.);
const ContaminantTransmissionCurve ContaminantDataServer::_oneCurve(1.);


ContaminantDataServer::ContaminantDataServer(
					 const vector<double> &obstime,
					 const vector<double> &cdepth,
					 const vector<double> &odepth,
					 const vector<double> &fdepth,
					 CalAtomCollection<ContaminantDataServer> *parent)
	: CalChild<CalAtomCollection<ContaminantDataServer> >(parent),
	  _time(obstime), _c_depth(cdepth), _o_depth(odepth), _f_depth(fdepth)
{
  //
  //    read the henke coefficient curves for carbon, oxygen and flourine
  //
  AbsorptionDataServer *ads = 0; ads = calServer.getAtom(ads);
  const AbsCoefCurve &f2C    = ads->henke("C").f2(),
    &f2O     = ads->henke("O").f2(),
    &f2F     = ads->henke("F").f2();

  // Get the atomic weight for each element
  double AC          = ads->atomicData("C").weight(), // [g/mol]
    AF          = ads->atomicData("F").weight(),
    AO          = ads->atomicData("O").weight(),

  // Get the atomic density for each element
    rhoC       = ads->atomicData("C").density() * 1.e-3,      // [g/cm^3]
    rhoO       = ads->atomicData("O").density() * 1.e-3,
    rhoF       = ads->atomicData("F").density() * 1.e-3;

  /* Set the energy grid from the Carbon energies */
  _eGrid = f2C.abscissa();
  unsigned Ne = _eGrid.size();

  /*
  cout<<"egrid: "<<_eGrid[100]<<" "<<_eGrid.size()<<endl;
  cout<<"Carbon: "<<f2C(_eGrid[100])<<endl;

  cout<<"CONSTANTS: "<<endl;
  cout<<"avogadro: "<<PhysicalConstants::avogadro<<endl;
  cout<<"r0: "<<PhysicalConstants::r0<<endl;
  cout<<"eVtimesAo: "<<PhysicalConstants::eVtimesAo<<endl;
  cout<<"rhoC AC: "<<rhoC<<" "<<AC<<endl;
  */

  /* Loop over each energy */
  for (unsigned n=0; n<Ne; ++n) {

     double &eV  = _eGrid[n]; // [eV]

     // Calculate the constant
     double wavelength_angstroms = 1.e-8 * PhysicalConstants::eVtimesAo / eV;

     double f0 = 2.0 * PhysicalConstants::avogadro *            // [1/mol]
                 PhysicalConstants::r0 * 1.e2 *           // [cm]
                 wavelength_angstroms * wavelength_angstroms; // [wavelength_ang]

     double numerator = 1.23984 / ( eV * f0);

     // Set the inverse mu values for each element at this energy
     _henke_inverse_mu_c.push_back( numerator / ( rhoC/AC*f2C(eV))); // Carbon
     _henke_inverse_mu_o.push_back( numerator / ( rhoO/AO*f2O(eV))); // Oxygen
     _henke_inverse_mu_f.push_back( numerator / ( rhoF/AF*f2F(eV))); // Flourine

     //cout<<"E, f0, f2C, inv_mu: "<<eV<<" "<<f0*rhoC/AC<<" "<<f2C(eV)<<" "<<_henke_inverse_mu_c[n]<<endl;
  }

} // ContaminantDataServer::ContaminantDataServer()

// Method: at(mission_time) - returns the transmission curve for the contaminant
//		     at a given mjd
//const ContaminantTransmissionCurve &ContaminantDataServer::at(int rev) const
ContaminantTransmissionCurve * ContaminantDataServer::at(double time) 
{
  
  // Find the nearest calibration point to the input time
   int ind=-999;
   for (unsigned n=0; n<_time.size()-1; ++n) {
       if (time >= _time[n] && time <= _time[n+1])
          ind=n;
   }

  // If failed - use the first or last calibrated time
   if (ind<0) {
      if (time < _time[0])
         ind=0;
      else 
         ind=_time.size()-1;

      errstr<<"Requested MJD "<<time<<" is outside the range "
            <<"for which the contamination has been calibrated. "
            <<"Using calibration for mission time "<<_time[ind]
            <<Sas::warning("NoContaminantForTime");
   }

   errstr<<"Using contamination depths (C,O,F): "<<_c_depth[ind]<<" "<<_o_depth[ind]
         <<" "<<  _f_depth[ind]<<" "<<ind<<Sas::message(Msg::App, Msg::Verbose);

  unsigned Ne = _eGrid.size();
  vector<double> trans(Ne);

   //cout<<"Ne "<<Ne<<" IND: "<<ind<<" "<<_henke_inverse_mu_c.size()<<endl;

  // Apply the contamination element depths for this time to
  // the individual Henke curves to produce a contamination curve
   for (unsigned n=0; n<Ne; ++n) {

       trans[n] = exp(-1.0 * _c_depth[ind] / _henke_inverse_mu_c[n])
                  * exp(-1.0 * _o_depth[ind] / _henke_inverse_mu_o[n])
                  * exp(-1.0 * _f_depth[ind] / _henke_inverse_mu_f[n]);

   //cout<<"G: "<<_eGrid[n]<<" "<<trans[n]<<" "<<_c_depth[ind]<<" "<<_henke_inverse_mu_c[n]<<endl;
   }


   ContaminantTransmissionCurve *totalContamination = 
                   new ContaminantTransmissionCurve(_eGrid, trans);

   return totalContamination;
} // ContaminantDataServer::at()


ContaminantDataServer::~ContaminantDataServer()
{
} // ContaminantDataServer::~ContaminantDataServer()

/* AI gcc4 porting
 * too few template-parameter-lists => template<>
 */
template<>
CalAtomCollection<ContaminantDataServer>::CalAtomCollection(
							const XmmInstrument *instrument,
							const Ccf *ccf,
							Xmm::Filter filter,
							AccuracyLevel accuracy)
{
	_filter		= filter;
	_accuracy	= accuracy;
	_nCCDs		= instrument->nCcds();
	_nNodes		= instrument->nNodes();
	_nEntries	= 1;
	ContaminantDataServer *null = 0;
	_collection = new vector<ContaminantDataServer *>(_nEntries, null);

   // Only valid for MOS so far - set to null explicitly for PN
	vector<double> obstime,cdepth, odepth, fdepth;

	if (instrument->id() == Xmm::EMOS1 || instrument->id() == Xmm::EMOS2)
	{
  	   CifEntry *e = ccf->entry(instrument, Ccf::Contamination);
	   DataSet *dataSet = e->open();

	  // Read time-dependent depth information
	   CcfConstituentTable ccftab(dataSet, "CONTAM_DEPTH");
	   const Table *table = ccftab.block();

      // Get time information
       CalOalUtility::fillVector(table, "TIME", obstime);

      // Get element depths as a function of time

       //CalOalUtility::fillVector(table, "H_DEPTH", hdepth);
       CalOalUtility::fillVector(table, "C_DEPTH", cdepth);
       //CalOalUtility::fillVector(table, "N_DEPTH", ndepth);
       CalOalUtility::fillVector(table, "O_DEPTH", odepth);
       CalOalUtility::fillVector(table, "F_DEPTH", fdepth);

       //cout<<"C_DEPTH[100] "<<obstime[1000]<<" "<<cdepth[1000]<<endl;

       dataSetServer->close(dataSet);
	}
	else { // PN - set to null
		obstime.push_back(1.0); obstime.push_back(100000.0);  // MJDs to cover full range of possible times
		cdepth.push_back(0.0); cdepth.push_back(0.0);
		odepth.push_back(0.0); odepth.push_back(0.0);
		fdepth.push_back(0.0); fdepth.push_back(0.0);
	}

	ContaminantDataServer *&cnds = (*_collection)[0];
	cnds = new ContaminantDataServer(obstime, cdepth,
                                             odepth, fdepth, this);

} // CalAtomCollection<ContaminantDataServer>::CalAtomCollection()


/* AI gcc4 porting
 * too few template-parameter-lists => template<>
 */
template<>
ContaminantDataServer *CalAtomCollection<ContaminantDataServer>::select(
					Ccd,
					Node,
					unsigned) const
{
	return (*_collection)[0];
} // CalAtomCollection<ContaminantDataServer>::select()

