// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997-2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	OmFlatFieldServer --- OM flatfield components server
//
//	DESCRIPTION:
//		Instances of class OmFlatFieldServer can give back maps
//		of the OM flatfield component that is associated with large-scale
//		sensitivity variations due to filter defects or contaminations
//		on the filter or the detector window. For more information see [2].
//
//	CHANGE HISTORY:
//		UL, 2000-01-26: 1.0.0: - initial version
//		UL, 2000-07-26: 1.1.0: - added method to access LED Template array
//
//	AUTHOR:
//		U. Lammers (UL), Wed Jan 26 11:27:09 MET 2000
//
//	REFERENCES:
//		[1]: Software Specification Document for the XMM Science Analysis
//			 Subsystem, M. Dahlem et. al.,
//		[2]: Calibration Access and Data Handbook, XMM instrument team
//
//	$Header: /sasbuild/sas_git_repo/cvs_copy/sasdev/sas/packages/cal/src/OmFlatFieldServer.h,v 1.7 2003/10/03 14:15:41 sasbuild Exp $
//

#ifndef OM_FLATFIELD_SERVER_H
#define OM_FLATFIELD_SERVER_H

#include "Ccf.h"
#include "CalChild.h"
#include "CalAtomsBase.h"
#include "CalDataVector.h"
#include "Image.h"
#include "Bivariate2DInterpolantRectangular.h"

//
//	---( OmFlatFieldServer )-------------------------------------------------
//
class OmFlatFieldServer : public CalAtom,
	  public virtual CalChild<CalAtomCollection<OmFlatFieldServer> > {
public:
	OmFlatFieldServer(double crvalx, double crpixx, double cdeltx,
					  double crvaly, double crpixy, double cdelty,
					  const SImage &largeScaleFlatField,
					  ValueError largeScaleFlatFieldAverage,
					  double largeScalePixelError,

					  const SImage &smallScaleFlatField,
					  ValueError smallScaleFlatFieldAverage,
					  double smallScalePixelError,

					  const SImage &ledTemplateMap,
					  ValueError ledTemplateAverage,
					  double ledTemplatePixelError,
					CalAtomCollection<OmFlatFieldServer> *parent = 0);
		//
		//	PURPOSE:
		//		Construct OmFlatFieldServer object.
		//
		//	ARGUMENTS:
		//		crvalx/y
		//		crpixx/y
		//		cdeltx/y	: numbers defining the grid points
		//						x_i = crvalx+(i-crpixx)*cdeltx
		//						x_i = crvaly+(i-crpixy)*cdelty
		//		large/small/ledTemplateMap
		//					: large/small/ledTemplate scale flatfield map
		//		large/small/ledTemplateAverage
		//					: large/small/ledTemplate scale flatfield value
		//					  averaged over all good detector pixels +
		//					  associated uncertainties
		//		large/small/ledTemplatePixelError
		//					: average uncertainty of single pixel [%]
		//


	static unsigned id() { return Ccf::LargeScaleSens; }
		//
		//	PURPOSE:
		//		Return a uniue ID which for mapping Cal atoms onto
		//		CCF constituents
		//


	virtual void dump(std::ostream &os);
		//
		//	PURPOSE:
		//		Dump data members to an ostream
		//
		//	ARGUMENTS:
		//		In:
		//			os:	stream to print on
		//


	virtual SImage *largeScaleFlatFieldMap(unsigned x0, unsigned y0,
										   unsigned dx, unsigned dy,
										   double lambda) const;
		//
		//	PURPOSE:
		//		Return the large-scale flatfield image in the PIXCOORD1
		//		window:
		//			[x0,x0+dx-1][y0,y0+dy-1]
		//		Please note: The CCF::LargeScaleSense constituent contains
		//					 the flatfield sampled on a much coarser grid;
		//					 2D interpolation is applied to compute values
		//					 for the inner mesh points.
		//
		//	ARGUMENTS:
		//		x0/y0/dx/dy	: parameters definining the lower left corner
		//					  and x/y extent of the flatfield image in
		//					  PIXCOORD1 to be returned.
		//		lambda		: wavelength (currently not used)
		//
		//	RETURNS:
		//		Pointer to requested large scale flatfield image - has to be
		//		deleted	after last use
		//


	virtual SImage *smallScaleFlatFieldMap(unsigned x0, unsigned y0,
										   unsigned dx, unsigned dy,
										   double countRateFrameRatio) const;
		//
		//	PURPOSE:
		//		Return the small-scale flatfield image in the PIXCOORD1
		//		window:
		//			[x0,x0+dx-1][y0,y0+dy-1]
		//
		//	ARGUMENTS:
		//		x0/y0/dx/dy	: parameters definining the lower left corner
		//					  and x/y extent of the flatfield image in
		//					  PIXCOORD1 to be returned.
		//		countRateFrameRatio
		//					: average ratio of count to framerate (currently
		//					  unused)
		//
		//	RETURNS:
		//		Pointer to requested small scale flatfield image - has to be
		//		deleted	after last use
		//


	virtual SImage *ledTemplateMap(unsigned x0, unsigned y0,
								   unsigned dx, unsigned dy) const;
		//
		//	PURPOSE:
		//		Return the LED template flatfield image in the PIXCOORD1
		//		window:
		//			[x0,x0+dx-1][y0,y0+dy-1]
		//
		//	ARGUMENTS:
		//		x0/y0/dx/dy	: parameters definining the lower left corner
		//					  and x/y extent of the flatfield image in
		//					  PIXCOORD1 to be returned.
		//
		//	RETURNS:
		//		Pointer to requested LED template flatfield image - has to be
		//		deleted	after last use
		//


	virtual ValueError largeScaleFlatFieldAverage() const;
	virtual ValueError smallScaleFlatFieldAverage() const;
	virtual ValueError ledTemplateAverage() const;
		//
		//	PURPOSE:
		//		Return large/small-scale flatfield averaged over all good
		//		detector pixels	plus associated uncertainty;
		//		ledTemplateAverage() returns average correction
		//		for nonflat illumination (see [2] for details)
		//


	virtual double largeScalePixelError() const;
	virtual double smallScalePixelError() const;
	virtual double ledTemplatePixelError() const;
		//
		//	PURPOSE:
		//		Return average uncertainty of single pixel large/small-scale
		//		flatfield value	in %.
		//


	virtual ~OmFlatFieldServer();
		//
		//	PURPOSE:
		//		Destruct OmFlatFieldServer object
		//

private:
	//	data members
	Slatec::Bivariate2DInterpolantRectangular<float> *_largeScaleFlatField;
	ValueError	_largeScaleFlatFieldAverage;
	double		_largeScalePixelError;

	SImage		*_smallScaleFlatField;
	ValueError	_smallScaleFlatFieldAverage;
	double		_smallScalePixelError;

	SImage		*_ledTemplateMap;
	ValueError	_ledTemplateAverage;
	double		_ledTemplatePixelError;
};

#endif
