// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997-2020 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	SWCXImageDataServer.cc --- implementation of class to return data from
//			     the Filter Wheel Closed data CCF
//
//	DESCRIPTION:
//
//	CHANGE HISTORY:
//		RDS, 22-11-19: 1.0.0: - original
//
//	AUTHOR:
//		R. Saxton, 2019-11-19
//
//	REFERENCES:
//		[1]: Software Specification Document for the XMM Science Analysis
//			 Subsystem, M. Dahlem et. al.,
//

#include "SWCXImageDataServer.h"
#include "CalOalUtility.h"
#include "CalServer.h"

#include <algorithm>
#include <cmath>

using namespace CalOalUtility;
using namespace std;

typedef multimap<const string,SImage>::iterator mapIter;

//
//	----------------------( mode parameters )-----------------------------
//

//  Define the SWCXImageDataServer class
//
SWCXImageDataServer::SWCXImageDataServer( 
        multimap<const string, SImage> swcxImage,
        CalAtomCollection<SWCXImageDataServer> *parent)
        : CalChild<CalAtomCollection<SWCXImageDataServer> >(parent), _swcxImage(swcxImage)
{
}
// Return the flare image for this combination
const SImage SWCXImageDataServer::getSWCXImage(string &line)
{
  SImage fim;

  // Return the image for this line
  mapIter s_it;
  pair<mapIter, mapIter> keyRange = _swcxImage.equal_range(line);

  // Test for match
  if (keyRange.first == _swcxImage.end() && 
      keyRange.second == _swcxImage.end()) 
        errstr<<"The CCF does not contain SWCX data "
        "for the line "<<line<<Sas::error("NoLineSwcxData");
      
  for (s_it = keyRange.first;  s_it != keyRange.second;  ++s_it)
        {
           fim = (*s_it).second;
        }
  
  //cout<<"Got: "<<line<<" "<<fim[300][149]<<endl;
  return fim;
}

template<>
CalAtomCollection<SWCXImageDataServer>::CalAtomCollection(
	                                     const XmmInstrument *instrument,
					     const Ccf *ccf,
					     Xmm::Filter filter,
					     AccuracyLevel accuracy)
{
	_filter		= filter;
	_accuracy	= accuracy;
	_nModes		= instrument->nModes();
        _nCCDs		= instrument->nCcds();

	SWCXImageDataServer *null = 0;
	_collection = new vector<SWCXImageDataServer *>(1, null);

	CifEntry *e = ccf->entry(instrument, Ccf::SWCX);
	DataSet *dataSet = e->open();

      // Open the flare image extension
        Table *table = dataSet->table(e->queryBlockName("SWCX"));

      // Read line and energy into vectors
        vector<string> line;
        vector<float> energy;
        //vector<vector<float> > vflareimg;
        //vector<unsigned long> extents = ad->parent()->size();
        //TImasterOffClosed = new SImage(extents[0], extents[1], ad->real32Data());
        ColumnData *insCol = table->column("INSTRUME")->data();
        CalOalUtility::fillVector(table, "LINE", line);
        CalOalUtility::fillVector(table, "ENERGY", energy);
        
        // Read the swcx image
        ColumnData *cd = table->column("SWCXMAP")->data();
        real32* fimgd = cd->real32Data();


        // Get current instrument as a string
        string instrumentName(Xmm::toString(instrument->id()));
        //fimgd[608400*3+780] = 3.0;
        //cout<<"Pix: "<<fimgd[608400*3+780]<<endl;
        
        //CalOalUtility::fillVector(table, "FLAREIMG", vflareimg);
        //vector<unsigned> extents = cd->size();
        //unsigned nx=extents[0], ny=extents[0];
        //FIND NX, NY SOMEHOW
        unsigned nx=780, ny=780; // Bit of a hack this

      // Produce a map with the key defined by the line for
      // the requested instrument
        multimap<const string, SImage> m;
   
        for (unsigned n=0; n<table->rows(); ++n) {
            string instId(uppercase(stringColumnRow(insCol, n)));
            if (instId == instrumentName) {

               SImage fim(nx,ny);

               int offset=n*nx*ny; // First pixel of the next map

               // Fill the image from the vector
               for (unsigned ix=0; ix<nx; ++ix)
                   for (unsigned iy=0; iy<ny; ++iy)
                      //fim[ix][iy] = fimgd[offset + nx*ix +iy];
                      fim[ix][iy] = fimgd[offset + ny*iy +ix];

               m.insert(make_pair(line[n], fim));
               //cout<<line[n]<<""<<fim[300][149]<<" "<<fim[149][300]<<endl;
            }
        }

      // Create the collection - just one
        (*_collection)[0] = new SWCXImageDataServer(m, this);

      // Delete the column pointers
      /*
        delete cdmodeId;
      */
	dataSetServer->close(dataSet);
} // CalAtomCollection<SWCXImageDataServer>::CalAtomCollection


/* AI gcc4 porting
 *  too few template-parameter-lists => template<>
 */
template<>
SWCXImageDataServer *CalAtomCollection<SWCXImageDataServer>::select(
					Ccd,
					Node,
					unsigned) const
{
        //unsigned n = index(modeOffset);

        if (!(*_collection)[0])
           errstr<<"The CCF does not contain SWCX data "
                          <<Sas::error("NoData");

	return (*_collection)[0];
} // CalAtomCollection<SWCXImageDataServer>::select
