// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997-2020 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	ScaleFactorServer.cc --- implementation of class to return data from
//			     the Filter Wheel Closed data CCF
//
//	DESCRIPTION:
//
//	CHANGE HISTORY:
//		RDS, 22-11-19: 1.0.0: - original
//
//	AUTHOR:
//		R. Saxton, 2019-11-19
//
//	REFERENCES:
//		[1]: Software Specification Document for the XMM Science Analysis
//			 Subsystem, M. Dahlem et. al.,
//

#include "ScaleFactorServer.h"
#include "CalOalUtility.h"
#include "CalServer.h"

#include <algorithm>
#include <cmath>

using namespace CalOalUtility;
using namespace std;

typedef multimap<const string,SImage>::iterator mapIter;

//
//	----------------------( mode parameters )-----------------------------
//

//  Define the ScaleFactorServer class
//
ScaleFactorServer::ScaleFactorServer( 
        multimap<const string, SImage> sfImage,
        CalAtomCollection<ScaleFactorServer> *parent)
        : CalChild<CalAtomCollection<ScaleFactorServer> >(parent), _sfImage(sfImage)
{
}
// Return the flare image for this combination
const SImage ScaleFactorServer::getScaleFactorImage(string &idxband)
{
  SImage fim;

  // Return the image for this band (Soft,Medium,Hard)
  mapIter s_it;
  pair<mapIter, mapIter> keyRange = _sfImage.equal_range(uppercase(idxband));

  // Test for match
  if (keyRange.first == _sfImage.end() && 
      keyRange.second == _sfImage.end()) 
        errstr<<"The CCF does not contain ScaleFactor data "
        "for the band "<<idxband<<Sas::error("NoScaleFactorData");
      
  for (s_it = keyRange.first;  s_it != keyRange.second;  ++s_it)
        {
           fim = (*s_it).second;
        }
  
  return fim;
}

template<>
CalAtomCollection<ScaleFactorServer>::CalAtomCollection(
	                                     const XmmInstrument *instrument,
					     const Ccf *ccf,
					     Xmm::Filter filter,
					     AccuracyLevel accuracy)
{
	_filter		= filter;
	_accuracy	= accuracy;
	_nModes		= instrument->nModes();
        _nCCDs		= instrument->nCcds();

	ScaleFactorServer *null = 0;
	_collection = new vector<ScaleFactorServer *>(1, null);

	CifEntry *e = ccf->entry(instrument, Ccf::SCALEFACTORS);
	DataSet *dataSet = e->open();

      // Open the flare image extension
        Table *table = dataSet->table(e->queryBlockName("SCALEFCS"));

      // Read band, filter and power-law index into vectors
        vector<string> idxband, filtFS;
        vector<float> poweridx;
        //vector<vector<float> > vflareimg;
        //vector<unsigned long> extents = ad->parent()->size();
        //TImasterOffClosed = new SImage(extents[0], extents[1], ad->real32Data());
        ColumnData *insCol = table->column("INSTRUME")->data();
        CalOalUtility::fillVector(table, "FILTER", filtFS);
        CalOalUtility::fillVector(table, "IDXBAND", idxband);
        CalOalUtility::fillVector(table, "POWERIDX", poweridx);
        
        // Read the swcx image
        ColumnData *cd = table->column("SCALEFCS")->data();
        real32* fimgd = cd->real32Data();


        // Get current instrument and filter as strings
        string instrumentName(Xmm::toString(instrument->id()));
        string filterName = uppercase(Xmm::toString(filter));
        unsigned nx=240, ny=240; // GET THIS FROM TDIM5 !!!!!

      // Produce a map with the key defined by the band for
      // the requested instrument and filter combination
        multimap<const string, SImage> m;
   
        for (unsigned n=0; n<table->rows(); ++n) {
            string instId(uppercase(stringColumnRow(insCol, n)));
            if (instId == instrumentName && 
                                uppercase(filtFS[n]) == filterName) {

               SImage fim(nx,ny);

               int offset=n*nx*ny; // First pixel of the next map

               // Fill the image from the vector
               for (unsigned ix=0; ix<nx; ++ix)
                   for (unsigned iy=0; iy<ny; ++iy)
                      //fim[ix][iy] = fimgd[offset + nx*ix +iy];
                      fim[ix][iy] = fimgd[offset + ny*iy +ix];

               m.insert(make_pair(uppercase(idxband[n]), fim));
            }
        }

      // Create the collection - just one
        (*_collection)[0] = new ScaleFactorServer(m, this);

      // Delete the column pointers
      /*
        delete cdmodeId;
      */
	dataSetServer->close(dataSet);
} // CalAtomCollection<ScaleFactorServer>::CalAtomCollection


/* AI gcc4 porting
 *  too few template-parameter-lists => template<>
 */
template<>
ScaleFactorServer *CalAtomCollection<ScaleFactorServer>::select(
					Ccd,
					Node,
					unsigned) const
{
        //unsigned n = index(modeOffset);

        if (!(*_collection)[0])
           errstr<<"The CCF does not contain ScaleFactor data "
                          <<Sas::error("NoData");

	return (*_collection)[0];
} // CalAtomCollection<ScaleFactorServer>::select
